﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectExcavator.DrawingObjects;
using ProjectExcavator.MovementStrategy;

namespace ProjectExcavator.Generics
{
    /// <summary>
    /// Класс для хранения коллекции
    /// </summary>
    internal class ExcavatorGenericStorage
    {
        /// <summary>
        /// Словарь (хранилище)
        /// </summary>
        readonly Dictionary<string, ExcavatorGenericCollection<DrawingExcavator,
        DrawingObjectExcavator>> _excavatorStorages;
        /// <summary>
        /// Возвращение списка названий наборов
        /// </summary>
        public List<string> Keys => _excavatorStorages.Keys.ToList();
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="pictureWidth"></param>
        /// <param name="pictureHeight"></param>
        public ExcavatorGenericStorage(int pictureWidth, int pictureHeight)
        {
            _excavatorStorages = new Dictionary<string,
            ExcavatorGenericCollection<DrawingExcavator, DrawingObjectExcavator>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        /// <summary>
        /// Добавление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void AddSet(string name)
        {
            if (!_excavatorStorages.ContainsKey(name))
            {
                ExcavatorGenericCollection<DrawingExcavator, DrawingObjectExcavator> newSet = new ExcavatorGenericCollection<DrawingExcavator, DrawingObjectExcavator>(_pictureWidth, _pictureHeight);
                _excavatorStorages.Add(name, newSet);
            }
        }
        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void DelSet(string name)
        {
            {
                if (_excavatorStorages.ContainsKey(name))
                {
                    _excavatorStorages.Remove(name);
                }
            }
        }
        /// <summary>
        /// Доступ к набору
        /// </summary>
        /// <param name="ind"></param>
        /// <returns></returns>
        public ExcavatorGenericCollection<DrawingExcavator, DrawingObjectExcavator>? this[string ind]
        {
            get
            {
                if (_excavatorStorages.ContainsKey(ind))
                {
                    return _excavatorStorages[ind];
                }
                return null;
            }
        }
    }
}
